@@ -504,22 +504,41 @@ class KerasConverter:
                     bn_params['bn_moving_variance'],
                     bn_params['bn_beta']]})
 
+    def insert_binary_op(
+            self,
+            layer,
+            op_type,
+            keras_graph_helper,
+            ncnn_graph_helper,
+            ncnn_helper):
+        ncnn_graph_attr = ncnn_helper.dump_args(
+            'BinaryOp', op_type=op_type, with_scalar=0)
+        inbounds = keras_graph_helper.get_node_inbounds(layer['layer']['name'])
+        last_node_name = inbounds[0]
+        for node_idx in range(1, len(inbounds)):
+            if node_idx == len(inbounds) - 1:
+                node_name = layer['layer']['name']
+            else:
+                node_name = layer['layer']['name']+'_slop_'+str(node_idx)
+            ncnn_graph_helper.node(
+                node_name,
+            [last_node_name, inbounds[node_idx]])
+            ncnn_graph_helper.set_node_attr(
+                node_name, {
+                    'type': 'BinaryOp', 'param': ncnn_graph_attr, 'binary': []})
+            last_node_name = node_name
+
     def Add_helper(
             self,
             layer,
             keras_graph_helper,
             ncnn_graph_helper,
             ncnn_helper):
-        ncnn_graph_attr = ncnn_helper.dump_args(
-            'BinaryOp', op_type=0, with_scalar=0)
-
-        ncnn_graph_helper.node(
-            layer['layer']['name'],
-            keras_graph_helper.get_node_inbounds(
-                layer['layer']['name']))
-        ncnn_graph_helper.set_node_attr(
-            layer['layer']['name'], {
-                'type': 'BinaryOp', 'param': ncnn_graph_attr, 'binary': []})
+        self.insert_binary_op(layer,
+            0,
+            keras_graph_helper,
+            ncnn_graph_helper,
+            ncnn_helper)
 
     def Multiply_helper(
             self,
@@ -527,16 +546,11 @@ class KerasConverter:
             keras_graph_helper,
             ncnn_graph_helper,
             ncnn_helper):
-        ncnn_graph_attr = ncnn_helper.dump_args(
-            'BinaryOp', op_type=2, with_scalar=0)
-
-        ncnn_graph_helper.node(
-            layer['layer']['name'],
-            keras_graph_helper.get_node_inbounds(
-                layer['layer']['name']))
-        ncnn_graph_helper.set_node_attr(
-            layer['layer']['name'], {
-                'type': 'BinaryOp', 'param': ncnn_graph_attr, 'binary': []})
+        self.insert_binary_op(layer,
+            2,
+            keras_graph_helper,
+            ncnn_graph_helper,
+            ncnn_helper)
 
     def Activation_helper(
             self,
@@ -570,7 +584,8 @@ class KerasConverter:
             else:
                 negative_slope = 0.0
 
-            if '_relu6' in layer['layer']['config']['activation']:
+            if '_relu6' in layer['layer']['config']['activation'] or \
+                    'relu6' in layer['layer']['config']['activation']:
                 layer['layer']['config']['max_value'] = 6.0
                 layer['layer']['config']['activation'] = 'relu'
 
@@ -795,7 +810,7 @@ class KerasConverter:
             ncnn_graph_helper,
             ncnn_helper):
 
-        SUPPORTED_ACTIVATION = ['', 'linear', 'softmax', 'hard_sigmoid']
+        SUPPORTED_ACTIVATION = ['', 'linear', 'softmax', 'hard_sigmoid', 'tanh']
         SUPPORTED_FUSED_ACTIVATION_TYPE = {
             'relu': 1,
             'sigmoid': 4
@@ -878,6 +893,45 @@ class KerasConverter:
                 keras_graph_helper.add_node_inbounds(
                     outbound_layer, layer['layer']['name'] + '_Softmax')
 
+        if layer['layer']['config']['activation'] == 'tanh':
+            ncnn_graph_attr = ncnn_helper.dump_args(
+                'InnerProduct',
+                num_output=num_output,
+                bias_term=1,
+                weight_data_size=weight_data_size)
+            ncnn_graph_helper.node(
+                layer['layer']['name'],
+                keras_graph_helper.get_node_inbounds(
+                    layer['layer']['name']))
+            ncnn_graph_helper.set_node_attr(
+                layer['layer']['name'], {
+                    'type': 'InnerProduct', 'param': ncnn_graph_attr, 'binary': [
+                        bn_params['bn_kernel'], bn_params['bn_bias']]})
+
+            outbound_layers = []
+
+            for name in keras_graph_helper.get_graph().keys():
+                for node in keras_graph_helper.get_graph()[
+                        name]['inbounds']:
+                    if layer['layer']['name'] == node:
+                        outbound_layers.append(name)
+
+            ncnn_graph_attr = ncnn_helper.dump_args('TanH')
+            ncnn_graph_helper.node(
+                layer['layer']['name'] + '_TanH', [layer['layer']['name'], ])
+            ncnn_graph_helper.set_node_attr(
+                layer['layer']['name'] + '_TanH', {
+                    'type': 'TanH', 'param': ncnn_graph_attr, 'binary': []})
+
+            keras_graph_helper.node(
+                layer['layer']['name'] + '_TanH', [layer['layer']['name'], ])
+
+            for outbound_layer in outbound_layers:
+                keras_graph_helper.remove_node_inbounds(
+                    outbound_layer, layer['layer']['name'])
+                keras_graph_helper.add_node_inbounds(
+                    outbound_layer, layer['layer']['name'] + '_TanH')
+
         if layer['layer']['config']['activation'] == 'hard_sigmoid':
             ncnn_graph_attr = ncnn_helper.dump_args(
                 'InnerProduct',
@@ -1050,14 +1104,15 @@ class KerasConverter:
         else:
             if(len(target_shape) == 3):
                 ncnn_graph_attr = ncnn_helper.dump_args(
-                    'Reshape', w=target_shape[1], h=target_shape[2])
+                    'Reshape', w=target_shape[1], h=target_shape[0], c=target_shape[2])
             else:
                 if(len(target_shape) == 2):
                     ncnn_graph_attr = ncnn_helper.dump_args(
-                        'Reshape', w=target_shape[1])
+                        'Reshape', w=target_shape[1], h=target_shape[0])
                 else:
                     if(len(target_shape) == 1):
-                        return
+                        ncnn_graph_attr = ncnn_helper.dump_args(
+                            'Reshape', w=target_shape[0])
                     else:
                         print(
                             '[ERROR] Reshape Layer Dim %d is not supported.' %
@@ -1077,6 +1132,50 @@ class KerasConverter:
             layer['layer']['name'], {
                 'type': 'Reshape', 'param': ncnn_graph_attr, 'binary': []})
 
+    def Permute_helper(
+            self,
+            layer,
+            keras_graph_helper,
+            ncnn_graph_helper,
+            ncnn_helper):
+        PERMUTE_LUT = {
+            # Dim=2
+            '1,2': (2, 0),
+            '2,1': (2, 1),
+
+            # Dim=3
+            '1,2,3': (3, 0),
+            '1,3,2': (3, 5),
+            '2,1,3': (3, 1),
+            '2,3,1': (3, 4),
+            '3,1,2': (3, 3),
+            '3,2,1': (3, 2),
+        }
+
+        dims = layer['layer']['config']['dims']
+        if(len(dims) in [2, 3]):
+            order_type = PERMUTE_LUT[','.join(list(map(str, dims)))]
+        else:
+            print(
+                '[ERROR] Permute Layer Dim [%s] is not supported.' %
+                str(dims))
+            frameinfo = inspect.getframeinfo(
+                inspect.currentframe())
+            print(
+                'Failed to convert at %s:%d %s()' %
+                (frameinfo.filename, frameinfo.lineno, frameinfo.function))
+            sys.exit(-1)
+
+        ncnn_graph_attr = ncnn_helper.dump_args(
+                'Permute', order_type=order_type)
+        ncnn_graph_helper.node(
+            layer['layer']['name'],
+            keras_graph_helper.get_node_inbounds(
+                layer['layer']['name']))
+        ncnn_graph_helper.set_node_attr(
+            layer['layer']['name'], {
+                'type': 'Permute', 'param': ncnn_graph_attr, 'binary': []})
+        
     def Cropping2D_helper(
             self,
             layer,
@@ -1216,16 +1315,11 @@ class KerasConverter:
             keras_graph_helper,
             ncnn_graph_helper,
             ncnn_helper):
-        ncnn_graph_attr = ncnn_helper.dump_args(
-            'BinaryOp', op_type=4, with_scalar=0)
-
-        ncnn_graph_helper.node(
-            layer['layer']['name'],
-            keras_graph_helper.get_node_inbounds(
-                layer['layer']['name']))
-        ncnn_graph_helper.set_node_attr(
-            layer['layer']['name'], {
-                'type': 'BinaryOp', 'param': ncnn_graph_attr, 'binary': []})
+        self.insert_binary_op(layer,
+            4,
+            keras_graph_helper,
+            ncnn_graph_helper,
+            ncnn_helper)
 
     def TensorFlowOpLayer_helper(
             self,
