@@ -1,5 +1,6 @@
 import inspect
 import sys
+import copy
 
 
 class NcnnParamDispatcher:
@@ -121,6 +122,9 @@ class NcnnParamDispatcher:
             4: {'output_width': 0},
         },
 
+        'TanH': {
+        },
+
         'Padding': {
             0: {'top': 0},
             1: {'bottom': 0},
@@ -163,6 +167,10 @@ class NcnnParamDispatcher:
             3: {'flag': 1}
         },
 
+        'Permute': {
+            0: {'order_type': 0}
+        },
+
         'Sigmoid': {
         },
 
@@ -171,7 +179,6 @@ class NcnnParamDispatcher:
         },
 
         'Split': {
-
         },
 
         'MemoryData': {
@@ -183,7 +190,7 @@ class NcnnParamDispatcher:
     }
 
     def dump_args(self, operator, **kwargs):
-        params = self.operation_param_table[operator]
+        params = copy.deepcopy(self.operation_param_table[operator])
         ncnn_args_phrase = ''
         for arg in params.keys():
             arg_name = list(params[arg].keys())[0]
@@ -204,7 +211,7 @@ class NcnnParamDispatcher:
                 ncnn_args_phrase = ncnn_args_phrase + \
                     '%d=%e ' % (arg, params_arg)
 
-            elif isinstance(params_arg, list):
+            elif isinstance(params_arg, (list, tuple)):
                 ncnn_args_phrase = ncnn_args_phrase + \
                     '%d=%d,%s ' % (-23300 - arg, len(params_arg),
                                    ','.join(list(map(str, params_arg))))
